@@ -116,15 +116,17 @@ Context_init (Context *self, PyObject *args, PyObject *kwds)
   PyObject *auth = NULL;
   int debug = 0;
   SMBCCTX *ctx;
+  char *proto = NULL;
   static char *kwlist[] =
     {
       "auth_fn",
       "debug",
+      "proto",
       NULL
     };
 
-  if (!PyArg_ParseTupleAndKeywords (args, kwds, "|Oi", kwlist,
-				    &auth, &debug))
+  if (!PyArg_ParseTupleAndKeywords (args, kwds, "|Ois", kwlist,
+				    &auth, &debug, &proto))
     {
       return -1;
     }
@@ -137,10 +139,20 @@ Context_init (Context *self, PyObject *args, PyObject *kwds)
 	  return -1;
 	}
 
-      Py_XINCREF (auth);
+      Py_INCREF (auth);
       self->auth_fn = auth;
     }
 
+#if SMBCLIENT_VERSION < 500 /* Older than 0.5.0 */
+  if(proto)
+  {
+    debugprintf ("-> Setting client max protocol to %s\n", proto);
+    lp_set_cmdline("client max protocol", proto);
+    debugprintf ("-> Setting client min protocol to %s\n", proto);
+    lp_set_cmdline("client min protocol", proto);
+  }
+#endif
+
   debugprintf ("-> Context_init ()\n");
 
   errno = 0;
@@ -158,6 +170,13 @@ Context_init (Context *self, PyObject *args, PyObject *kwds)
   smbc_setOptionUserData (ctx, self);
   if (auth)
     smbc_setFunctionAuthDataWithContext (ctx, auth_fn);
+#if SMBCLIENT_VERSION >= 500 /* 0.5.0 or newer */
+  if(proto)
+  {
+    debugprintf ("-> Setting client min/max protocol to %s\n", proto);
+    smbc_setOptionProtocols (ctx, proto, proto);
+  }
+#endif
 
   if (smbc_init_context (ctx) == NULL)
     {
@@ -180,6 +199,7 @@ Context_dealloc (Context *self)
       smbc_free_context (self->context, 1);
     }
 
+  Py_XDECREF (self->auth_fn);
   Py_TYPE(self)->tp_free ((PyObject *) self);
 }
 
@@ -381,12 +401,15 @@ Context_opendir (Context *self, PyObject *args)
     {
       smbc_DirType.tp_dealloc (dir);
       debugprintf ("%p <- Context_opendir() EXCEPTION\n", self->context);
-      return NULL;
+      dir = NULL;
     }
+	else
+	  {
+			debugprintf ("%p <- Context_opendir() = Dir\n", self->context);
+	  }
 
   Py_DECREF (largs);
   Py_DECREF (lkwlist);
-  debugprintf ("%p <- Context_opendir() = Dir\n", self->context);
   return dir;
 }
 
@@ -875,6 +898,34 @@ Context_setTimeout (Context *self, PyObject *value, void *closure)
   return 0;
 }
 
+static PyObject *
+Context_getPort (Context *self, void *closure)
+{
+  int port = smbc_getPort (self->context);
+  return PyLong_FromLong (port);
+}
+
+static int
+Context_setPort (Context *self, PyObject *value, void *closure)
+{
+#if PY_MAJOR_VERSION < 3
+  if (!PyInt_Check (value))
+#else
+  if (!PyLong_Check (value))
+#endif
+    {
+      PyErr_SetString (PyExc_TypeError, "must be long");
+      return -1;
+    }
+
+#if PY_MAJOR_VERSION < 3
+  smbc_setPort (self->context, PyInt_AsLong (value));
+#else
+  smbc_setPort (self->context, PyLong_AsLong (value));
+#endif
+  return 0;
+}
+
 static int
 Context_setFunctionAuthData (Context *self, PyObject *value, void *closure)
 {
@@ -884,7 +935,8 @@ Context_setFunctionAuthData (Context *self, PyObject *value, void *closure)
       return -1;
     }
 
-  Py_XINCREF (value);
+  Py_XDECREF (self->auth_fn);
+  Py_INCREF (value);
   self->auth_fn = value;
   smbc_setFunctionAuthDataWithContext (self->context, auth_fn);
   return 0;
@@ -912,6 +964,28 @@ Context_setOptionDebugToStderr (Context *self, PyObject *value,
   return 0;
 }
 
+static PyObject *
+Context_getOptionFullTimeNames (Context *self, void *closure)
+{
+  smbc_bool b;
+  b = smbc_getOptionFullTimeNames (self->context);
+  return PyBool_FromLong ((long) b);
+}
+
+static int
+Context_setOptionFullTimeNames (Context *self, PyObject *value,
+				void *closure)
+{
+  if (!PyBool_Check (value))
+    {
+      PyErr_SetString (PyExc_TypeError, "must be Boolean");
+      return -1;
+    }
+
+  smbc_setOptionFullTimeNames (self->context, value == Py_True);
+  return 0;
+}
+
 static PyObject *
 Context_getOptionNoAutoAnonymousLogin (Context *self, void *closure)
 {
@@ -1002,6 +1076,12 @@ PyGetSetDef Context_getseters[] =
       (getter) Context_getTimeout,
       (setter) Context_setTimeout,
       "Get the timeout used for waiting on connections and response data(in milliseconds)",
+      NULL },
+
+		{ "port",
+      (getter) Context_getPort,
+      (setter) Context_setPort,
+      "Set the TCP port used to connect (0 means default).",
       NULL },
 
     { "functionAuthData",
@@ -1014,6 +1094,12 @@ PyGetSetDef Context_getseters[] =
       (getter) Context_getOptionDebugToStderr,
       (setter) Context_setOptionDebugToStderr,
       "Whether to log to standard error instead of standard output.",
+      NULL },
+
+		{ "optionFullTimeNames",
+      (getter) Context_getOptionFullTimeNames,
+      (setter) Context_setOptionFullTimeNames,
+      "Use full time names (Create Time)",
       NULL },
 
     { "optionNoAutoAnonymousLogin",
@@ -1150,7 +1236,7 @@ PyMethodDef Context_methods[] =
 "                  to names.  Without the plus sign, SIDs are not mapped;\n"
 "                  rather they are simply converted to a string format.\n"
       "@return: a string representing the actual extended attributes of the uri" },
-      
+
        { "setxattr",
       (PyCFunction) Context_setxattr, METH_VARARGS,
       "setxattr(uri, the_acl) -> int\n\n"
